<?php
/**
 * Freemius Extension
 *
 * @package NotificationX\Extensions
 */

namespace NotificationXPro\Extensions\Freemius;

use NotificationX\GetInstance;
use NotificationX\Core\Rules;
use NotificationX\Extensions\Extension;
use NotificationX\Extensions\Freemius\FreemiusConversions as FreemiusConversionsFree;

/**
 * Freemius Extension
 */
class FreemiusConversions extends FreemiusConversionsFree {
    use Freemius;
    public $cron_schedule = 'nx_freemius_interval';
    
    /**
     * Initially Invoked when initialized.
     */
    public function __construct(){
        parent::__construct();
        $this->construct();
        $this->freemius_id = $this->id;

        // add_action( 'nx_settings_saved', [Helper::get_instance(), 'freemius_connect'] );
        add_filter('nx_notification_template', [$this, 'freemius_custom_fields'], 5);
    }

     /**
     * This method is an implementable method for All Extension coming forward.
     *
     * @param array $args Settings arguments.
     * @return mixed
     */
    public function freemius_custom_fields($template) {
        $template["freemius_fifth_param"] = [
            'name'     => "freemius_fifth_param",
            'type'     => "select",
            'priority' => 26,
            'default'  => 'tag_plan_title',
            'options'  => [
                'tag_custom' => [
                    'value' => 'tag_custom',
                    'label' => __('Custom', 'notificationx-pro'),
                    'rules' => Rules::includes( 'source', ['freemius_conversions'] ),
                ],
                'tag_plan_title' => [
                    'value' => 'tag_plan_title',
                    'label' => __('Plan Title ', 'notificationx-pro'),
                    'rules' => Rules::includes( 'source', ['freemius_conversions'] ),
                ]
            ],
            'rules' => Rules::includes( 'source', ['freemius_conversions'] ),
        ];
        $template["custom_freemius_fifth_param"] = [
            'name'     => "custom_freemius_fifth_param",
            'type'     => "text",
            'priority' => 27,
            'default'  => __('Pro Plus', 'notificationx-pro'),
            'rules'    => Rules::is( 'notification-template.freemius_fifth_param', 'tag_custom' ),
        ];
        $template["freemius_sixth_param"] = [
            'name'     => "freemius_sixth_param",
            'type'     => "text",
            'priority' => 28,
            'default'  => __('Subscription For', 'notificationx-pro'),
            'rules'    => Rules::includes('source', [ 'freemius_conversions' ]),
        ];
        $template["freemius_seventh_param"] = [
            'name'     => "freemius_seventh_param",
            'type'     => "select",
            'priority' => 29,
            'default'  => 'tag_pricing',
            'options'  => [
                'tag_custom' => [
                    'value' => 'tag_custom',
                    'label' => __('Custom', 'notificationx-pro'),
                    'rules' => Rules::includes( 'source', ['freemius_conversions'] ),
                ],
                'tag_pricing' => [
                    'value' => 'tag_pricing',
                    'label' => __('Price', 'notificationx-pro'),
                    'rules' => Rules::includes( 'source', ['freemius_conversions'] ),
                ]
            ],
            'rules'    => Rules::includes('source', [ 'freemius_conversions' ]),
        ];
        $template["custom_freemius_seventh_param"] = [
            'name'     => "custom_freemius_seventh_param",
            'type'     => "text",
            'priority' => 29.5,
            'default'  => __('99 USD/Year', 'notificationx-pro'),
            'rules'    => Rules::is( 'notification-template.freemius_seventh_param', 'tag_custom' ),
        ];
        return $template;
    }

    public function source_error_message($messages) {
        if ( empty( $this->dev_id ) || empty( $this->dev_public_key ) || empty( $this->dev_secret_key )) {
            $url = admin_url('admin.php?page=nx-settings&tab=tab-api-integrations#freemius_settings_section');
            $messages[$this->id] = [
                'message' => sprintf( '%s <a href="%s" target="_blank">%s</a>.',
                    __( 'You have to setup your Dev ID, Public Key, Secret Key from', 'notificationx-pro' ),
                    $url,
                    __( 'settings', 'notificationx-pro' )
                ),
                'html' => true,
                'type' => 'error',
                'rules' => Rules::is('source', $this->id),
            ];
        }
        return $messages;
    }

    public function init(){
        parent::init();
    }

    public function fallback_data( $data, $saved_data, $settings ){
        $data['title'] = __('Anonymous', 'notificationx-pro');
        return $data;
    }


    public function get_conversions($item_id, $type = '', $plugin_name = '') {
        if (!$item_id) {
            return [];
        }
        $subscriptions = $this->freemius()->Api("/plugins/$item_id/subscriptions.json");
        $users         = $this->freemius()->Api("/plugins/$item_id/users.json");
        $pricing       = $this->freemius()->Api("/plugins/$item_id/pricing.json");
        return Helper::get_instance()->get_sales_data($subscriptions, $users, $pricing, $plugin_name);
    }
}
