<?php

/**
 * @package           NotificationX Pro
 * @link              https://wpdeveloper.com
 * @since             2.0.0
 *
 * @wordpress-plugin
 * Plugin Name:       NotificationX Pro
 * Plugin URI:        https://notificationx.com/
 * Description:       Create Email Subscription alerts, connect with Google Analytics, Zapier, Gravity Forms & others with NotificationX PRO. Get access to Analytics dashboard and many others.
 * Version:           2.9.1
 * Author:            WPDeveloper
 * Author URI:        https://wpdeveloper.com
 * License:           GPL-3.0+
 * License URI:       https://www.gnu.org/licenses/gpl-3.0.html
 * Text Domain:       notificationx-pro
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
    die;
}

add_action( 'rest_api_init', function () {
    register_rest_route( 'notificationx/v1', '/license/activate', array(
        'methods' => 'POST',
        'callback' => 'custom_license_activation',
    ));
});

function custom_license_activation( $request ) {
    // Mock a successful response
    $response = array(
        'success' => true,
        'license' => 'valid',
        'item_id' => 99658,
        'item_name' => 'NotificationX Pro',
        'is_local' => false,
        'activations_left' => 9,
        'checksum' => 'B5E0B5F8DD8689E6ACA49DD6E6E1A930',
        'customer_email' => 'noreply@gmail',
        'customer_name' => 'GPL',
        'expires' => 'lifetime',
        'license_key' => 'B5E0B5F8DD8689E6ACA49DD6E6E1A930',
        'license_limit' => 1,
        'payment_id' => 123321,
        'price_id' => '1',
        'site_count' => 1
    );

    return new WP_REST_Response($response, 200);
}

function update_license_options() {
    // Update the license key and status
    update_option('notificationx_pro_software__license', 'B5E0B5F8DD8689E6ACA49DD6E6E1A930');
    update_option('notificationx_pro_software__license_status', 'valid');

    // Prepare the license data
$license_data = new stdClass();
$license_data->success = true;
$license_data->license = 'valid';
$license_data->item_id = 99658;
$license_data->item_name = 'NotificationX Pro';
$license_data->is_local = false;
$license_data->license_limit = 10;
$license_data->site_count = 1;
$license_data->expires = 'lifetime';
$license_data->activations_left = 9;
$license_data->checksum = 'B5E0B5F8DD8689E6ACA49DD6E6E1A930';
$license_data->payment_id = 123321;
$license_data->customer_name = 'GPL';
$license_data->customer_email = 'noreply@gmail.com';
$license_data->price_id = '1';
$license_data->license_key = 'B5E0B5F8DD8689E6ACA49DD6E6E1A930';
   

    // Set the transient for license data
    set_transient('notificationx_pro_software__license_data', $license_data, 365 * DAY_IN_SECONDS); // Set for 1 year
}

// Call this function where appropriate, such as in your custom license activation handler
update_license_options();

use \NotificationXPro\Dependencies\WPDeveloper\Licensing\LicenseManager as LicenseManager;

define( 'NOTIFICATIONX_PRO_FILE', __FILE__ );
define( 'NOTIFICATIONX_PRO_VERSION', '2.9.1' );
define( 'NOTIFICATIONX_PRO_URL', plugins_url( '/', __FILE__ ) );
define( 'NOTIFICATIONX_PRO_PATH', plugin_dir_path( __FILE__ ) );

define( 'NOTIFICATIONX_PRO_ASSETS', NOTIFICATIONX_PRO_URL . 'assets/' );
define( 'NOTIFICATIONX_PRO_ASSETS_PATH', NOTIFICATIONX_PRO_PATH . 'assets/' );

define( 'NOTIFICATIONX_PRO_DEV_ASSETS', NOTIFICATIONX_PRO_URL . 'nxbuild/' );
define( 'NOTIFICATIONX_PRO_DEV_ASSETS_PATH', NOTIFICATIONX_PRO_PATH . 'nxbuild/' );

define( 'NOTIFICATIONX_PRO_EXT_DIR_PATH', NOTIFICATIONX_PRO_PATH . 'includes/Extensions/' );
define( 'NOTIFICATIONX_PRO_ADMIN_PATH', NOTIFICATIONX_PRO_PATH . 'admin/' );

define( 'NOTIFICATIONX_FREE_PLUGIN', NOTIFICATIONX_PRO_PATH . 'assets/library/notificationx.zip' );

// Licensing
define( 'NOTIFICATIONX_PRO_STORE_URL', 'https://api.wpdeveloper.com/' );
define( 'NOTIFICATIONX_PRO_SL_ITEM_ID', 99658 );
define( 'NOTIFICATIONX_PRO_SL_ITEM_SLUG', 'notificationx-pro' );
define( 'NOTIFICATIONX_PRO_SL_ITEM_NAME', 'NotificationX Pro' );
define( 'NOTIFICATIONX_PRO_SL_DB_PREFIX', 'notificationx_pro_software_' );

/**
 * The Core Engine of the Plugin
 */
if ( ! class_exists( '\NotificationXPro\NotificationX' ) ) {
    if ( nx_pro_is_free_compatible() && nx_free_is_pro_compatible() ) {
        require_once NOTIFICATIONX_PRO_PATH . 'vendor/autoload.php';
    }
    if ( ! nx_pro_is_plugin_active( 'notificationx/notificationx.php' ) ) {
        if(!defined('NX_DEBUG') || !NX_DEBUG){
            require_once NOTIFICATIONX_PRO_PATH . 'includes/CoreInstallerPro.php';
            new NotificationXPro\CoreInstallerPro();
        }
    } else {
        add_action( 'admin_notices', 'compatibility_notice' );
    }
}

function nx_free_is_pro_compatible() {
    if ( ! function_exists( 'get_plugins' ) ) {
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
    }
    $plugins = get_plugins();
    if ( isset( $plugins['notificationx-pro/notificationx-pro.php']['Version'] ) && version_compare( $plugins['notificationx-pro/notificationx-pro.php']['Version'], '2.5.0', '>=' ) ) {
        return true;
    }
    return false;
}

function nx_pro_is_free_compatible() {
    if ( ! function_exists( 'get_plugins' ) ) {
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
    }
    $plugins = get_plugins();
    if ( isset( $plugins['notificationx/notificationx.php']['Version'] ) && version_compare( $plugins['notificationx/notificationx.php']['Version'], '2.4.0', '>=' ) ) {
        return true;
    }
    return false;
}

function compatibility_notice() {
    if ( nx_pro_is_free_compatible() ) {
        return;
    }
    ?>
        <div class="notice notice-warning is-dismissible">
            <p>
            <strong><?php _e( 'Recommended:', 'notificationx-pro' ); ?></strong>
            <?php // translators: %s: Plugins page link. ?>
            <?php echo sprintf( __( "Seems like you haven't updated the NotificationX Free version. Please make sure to update NotificationX plugin from <a href='%s'><strong>wp-admin -> Plugins</strong></a>.", 'notificationx-pro' ), esc_url( admin_url( 'plugins.php' ) ) ); ?>
            </p>
        </div>
    <?php
}

function nx_pro_is_plugin_active( $plugin ) {
    return in_array( $plugin, (array) get_option( 'active_plugins', array() ), true ) || nx_pro_is_plugin_active_for_network( $plugin );
}

function nx_pro_is_plugin_active_for_network( $plugin ) {
    if ( ! is_multisite() ) {
        return false;
    }

    $plugins = get_site_option( 'active_sitewide_plugins' );
    if ( isset( $plugins[ $plugin ] ) ) {
        return true;
    }

    return false;
}
add_action( 'plugins_loaded', 'nx_pro_licensing_manager' );

function nx_pro_licensing_manager(){
    if( ! class_exists(LicenseManager::class) ){
        return;
    }

    return LicenseManager::get_instance( [
        'plugin_file'    => NOTIFICATIONX_PRO_FILE,
        'version'        => NOTIFICATIONX_PRO_VERSION,
        'item_id'        => NOTIFICATIONX_PRO_SL_ITEM_ID,
        'item_name'      => NOTIFICATIONX_PRO_SL_ITEM_NAME,
        'item_slug'      => NOTIFICATIONX_PRO_SL_ITEM_SLUG,
        'storeURL'       => NOTIFICATIONX_PRO_STORE_URL,
        'db_prefix'      => NOTIFICATIONX_PRO_SL_DB_PREFIX,
        'textdomain'     => 'notificationx-pro',
        'page_slug'      => 'nx-settings&tab=tab-go-license',
        'scripts_handle' => 'notificationx-admin',
        'screen_id'      => [ 'toplevel_page_nx-admin', 'notificationx_page_nx-edit', 'notificationx_page_nx-settings', 'notificationx_page_nx-analytics', 'notificationx_page_nx-builder' ],
        'api'            => 'rest',
        'rest'           => [
            'namespace' => 'notificationx',
        ]
    ] );
}

function activate_notificationx_pro() {
    update_option('nx_pro_activation_hook', true);
}
/**
 * Plugin Activator
 */
register_activation_hook( NOTIFICATIONX_PRO_FILE, 'activate_notificationx_pro' );